import math
import xvg.data
import xvg.math
import xvg.models
from .context import Context


class Interpreter():

    def __init__(self):
        self.arguments = {}

    def interpret(self, context, script):
        """ Evaluates a script string with a context """
        exec(script,
             self.createGlobalsMap(context),
             self.createLocalsMap(context))

    def interpretFile(self, context, path):
        """ Evaluates a script file with a context """
        with open(path) as file:
            self.run(file.read())

    def createGlobalsMap(self, context):
        return {
            "__builtins__": {"locals": locals, "globals": globals}
        }

    def createLocalsMap(self, context):
        localsMap = {}

        localsMap['math'] = math
        localsMap['debug'] = print

        localsMap.update(xvg.data.getModuleClasses(xvg.data))
        localsMap.update(xvg.data.getPublicModuleFunctions(xvg.data))

        localsMap.update(xvg.data.getModuleClasses(xvg.math))
        localsMap.update(xvg.data.getPublicModuleFunctions(xvg.math))

        localsMap.update(xvg.data.getModuleClasses(xvg.models))
        localsMap.update(xvg.data.getPublicModuleFunctions(xvg.models))

        localsMap.update(context.getPublicFields())
        localsMap.update(context.getPublicMethods())

        return localsMap



from .ellipse import Ellipse
from .matrix import Matrix
from .polygon import Polygon, Regular3gon, Regular4gon, Regular5gon
from .random import randomReal, randomWhole, randomGaussian
from .rectangle import Rectangle
from .spline import Spline
from .vector import Vector

__all__ = [
    'Ellipse',
    'Matrix',
    'Polygon', 'Regular3gon', 'Regular4gon', 'Regular5gon',
    'randomReal', 'randomWhole', 'randomGaussian',
    'Rectangle',
    'Spline',
    'Vector'
]



from xvg.data import toJSON


class Renderer():

    def render(self, context):
        """ Renders the image and returns the result """
        return toJSON(context)

    def renderFile(self, context, path):
        """ Renders the image and writes the result file """
        with open(path, "w") as file:
            file.write(self.render(context))



from .renderer import Renderer
from .context import Context
from .interpreter import Interpreter


class Engine():

    def __init__(self, renderer=None, context=None, interpreter=None):
        self.renderer = renderer or Renderer()
        self.context = context or Context()
        self.interpreter = interpreter or Interpreter()

    def process(self, script):
        self.interpreter.interpret(self.context, script)
        return self.renderer.render(self.context)



class Polygon():
    def __init__(self):
        pass


class Regular3gon():
    def __init__(self):
        pass


class Regular4gon():
    def __init__(self):
        pass


class Regular5gon():
    def __init__(self):
        pass



from json import loads, dumps
from .object import Object


def toJSON(data):
    return dumps(data, default=lambda o: o.__dict__)


def fromJSON(data):
    return loads(data, object_hook=lambda d: Object(d))



from xvg.data import Object
from xvg.math import Vector


class Context(Object):

    def __init__(self):
        self.size = Vector()
        self.scale = Vector()

    def saveModel(self, id, model):
        pass

    def loadModel(self, id):
        pass

    def drawModel(self, id):
        pass
